
# cartwidget.py -- widget used in the cartesion coord editor tab
#
# Copyright (c) Adam Tenderholt, Stanford University, 2005-2006
#                               a-tenderholt@stanford.edu
#
# This program is free software; you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version with the exceptions noted in the 
# Help dialog.

import sys
from os.path import exists
from math import sin, cos

from PyQt4 import QtGui,QtCore
from cclib.parser import ccopen, utils
import openbabel

from glwidget import *
from customdelegate import *
from cartmodels import *

table = utils.PeriodicTable()

class CartWidget(QtGui.QWidget):
    
    def __init__(self, parent = None, flags = 0):
        QtGui.QWidget.__init__(self)

        self.setObjectName("cartWidget")

        self.gridlayout = QtGui.QGridLayout(self)
        self.gridlayout.setMargin(9)
        self.gridlayout.setSpacing(6)
        self.gridlayout.setObjectName("gridlayout")

        self.splitter = QtGui.QSplitter(self)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter3")

        self.glWidget = GLWidget(self.splitter)

        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Maximum,QtGui.QSizePolicy.Policy(7))
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.glWidget.sizePolicy().hasHeightForWidth())

        self.rightWidget = QtGui.QWidget(self.splitter)
        self.rightWidget.setObjectName("rightWidget")

        self.vboxlayout = QtGui.QVBoxLayout(self.rightWidget)
        self.vboxlayout.setMargin(0)
        self.vboxlayout.setSpacing(6)
        self.vboxlayout.setObjectName("vboxlayout")

        self.hboxlayout = QtGui.QHBoxLayout()
        self.hboxlayout.setMargin(0)
        self.hboxlayout.setSpacing(6)
        self.hboxlayout.setObjectName("hboxlayout")

        self.label = QtGui.QLabel(self.rightWidget)
        self.label.setObjectName("label")
        self.label.setText("Optimization Step:")
        self.hboxlayout.addWidget(self.label)

        self.spinBox = QtGui.QSpinBox(self.rightWidget)
        self.spinBox.setObjectName("spinBox")
        self.hboxlayout.addWidget(self.spinBox)

        spacerItem = QtGui.QSpacerItem(41,20,QtGui.QSizePolicy.Expanding,QtGui.QSizePolicy.Minimum)
        self.hboxlayout.addItem(spacerItem)

        self.checkBox = QtGui.QCheckBox(self.rightWidget)
        self.checkBox.setObjectName("checkBox")
        self.checkBox.setText("Lock")
        self.checkBox.setCheckState(QtCore.Qt.Checked)
        self.hboxlayout.addWidget(self.checkBox)
        self.vboxlayout.addLayout(self.hboxlayout)

        self.cartView = QtGui.QTableView(self.rightWidget)
        self.cartModel = OBMolModel()
        self.cartView.setModel(self.cartModel)
        self.cartView.show()

        self.customDelegate = CustomDelegate()
        self.selectionModel = QtGui.QItemSelectionModel(self.cartModel)
        self.cartView.setSelectionModel(self.selectionModel)

        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Minimum,QtGui.QSizePolicy.Policy(7))
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cartView.sizePolicy().hasHeightForWidth())

        self.cartView.setSizePolicy(sizePolicy)
        self.cartView.setObjectName("cartView")
        self.vboxlayout.addWidget(self.cartView)
        self.gridlayout.addWidget(self.splitter,0,0,1,1)

        self.splitter.setSizes([415,185])
      
#slots
        QtCore.QObject.connect(self.spinBox,QtCore.SIGNAL("valueChanged(int)"),self.slotUpdateGeometry)
        QtCore.QObject.connect(self.checkBox,QtCore.SIGNAL("stateChanged(int)"),self.glWidget.slotEnableLock)
        QtCore.QObject.connect(self.selectionModel,QtCore.SIGNAL("selectionChanged(const QItemSelection&, const QItemSelection&)"), self.changeCartSelection)
        QtCore.QObject.connect(self.glWidget,QtCore.SIGNAL("signalRotateMolecule"), self.cartModel.rotateMolecule)
        QtCore.QObject.connect(self.cartModel,QtCore.SIGNAL("dataChanged(const QModelIndex&, const QModelIndex&)"),self.glWidget.slotDataChanged)

#class variables
        self.data = None

    def setData(self,data):
        self.data = data

        if not hasattr(data, "atomcoords"):
            return

        if not hasattr(data, "atomnos"):
            return

        size = len(data.atomcoords)
        self.spinBox.setMaximum(size - 1)
        
        molecule = makeopenbabel(data.atomcoords[0],data.atomnos)

        self.cartModel.setMolecule(molecule)
        self.cartView.setModel(self.cartModel)
        self.glWidget.obmol = self.cartModel.obmol

        self.cartView.setColumnWidth(0,55)
        self.cartView.setColumnWidth(1,65)
        self.cartView.setColumnWidth(2,65)
        self.cartView.setColumnWidth(3,65)
        self.cartView.setItemDelegate(self.customDelegate)

    def setMolecule(self, molecule):

        self.cartModel.setMolecule(molecule)
        self.cartView.setModel(self.cartModel)
        self.glWidget.obmol = self.cartModel.obmol

        self.cartView.setColumnWidth(0,55)
        self.cartView.setColumnWidth(1,65)
        self.cartView.setColumnWidth(2,65)
        self.cartView.setColumnWidth(3,65)
        self.cartView.setItemDelegate(self.customDelegate)

    def slotUpdateGeometry(self, value):

        if not self.data:
            return

        molecule = makeopenbabel(self.data.atomcoords[value], self.data.atomnos)
        self.cartModel.setMolecule(molecule)
        self.glWidget.obmol = self.cartModel.obmol
        self.glWidget.updateGL()
        self.spinBox.setValue(value)

        self.emit(QtCore.SIGNAL("signalUpdateGeometry(int)"),value)

    def changeCartSelection(self, current, previous):
        indexes = self.selectionModel.selectedIndexes()
        dict = {}
        for index in indexes:
            dict[index.row()] = 1
        self.glWidget.setSelected(dict.keys())

    def slotUpdateCartData(self,topLeft,bottomRight):
        row = topLeft.row()
        col = topLeft.column()

        if row != bottomRight.row() or col != bottomRight.column():
            print "There's an error with editing Cartesian data"
            return

        item = self.cartModel.item(row,col)
        if col == 0:
            tempdata = item.data(QtCore.Qt.DisplayRole).toString()
            data = table.number[str(tempdata)]

        else:
            data, ok = item.data(QtCore.Qt.DisplayRole).toDouble()

        self.glWidget.updateMolecule(row,col,data)

    def addAtom(self):
        self.cartModel.addAtom()
        self.glWidget.obmol = self.cartModel.obmol
        self.cartView.setColumnWidth(0,55)
        self.cartView.setColumnWidth(1,65)
        self.cartView.setColumnWidth(2,65)
        self.cartView.setColumnWidth(3,65)
        self.cartView.setItemDelegate(self.customDelegate)

    def removeAtom(self):
        indexes = self.selectionModel.selectedIndexes()

        #get indices and sort
        rows = []
        for index in indexes:
            rows.append(index.row())
        rows.sort()

        i = 0
        for row in rows:
            self.cartModel.removeAtom(row - i)
            i += 1


if __name__=='__main__':
    app = QtGui.QApplication(sys.argv)
    window = CartWidget()
    window.resize(700,500)
    window.show()
    button = QtGui.QPushButton("foo")
    button.show()
    QtCore.QObject.connect(button, QtCore.SIGNAL("clicked()"), window.addAtom)

    if len(sys.argv)==2 and exists(sys.argv[1]):
        parser = ccopen(sys.argv[1])
        data = parser.parse()
        window.setData(data) 

    sys.exit(app.exec_())

