from math import cos, acos, sin, sqrt, pi
from cclib.parser import utils
import numpy

table = utils.PeriodicTable()
xvec = numpy.array([1, 0, 0])

class IntModel(object):
    def __init__(self):
        self.matrix = []
        self.obmol = None
        self.names = []
        self.numnames = []

    def setMolecule(self, molecule):
        self.obmol = molecule
        self.matrix = self.calculateZMatrix(molecule)

        size = molecule.NumAtoms()
        for i in range(size):
            atom = molecule.GetAtom(i+1)
            num = atom.GetAtomicNum()
            name = table.element[num]
            self.names.append(name)
            self.numnames.append("(%i) %s"%(i+1, name))

    def calculateZMatrix(self,obmol):
        size = obmol.NumAtoms()
        matrix = [[]]
        
        for i in range(1,size):

            current = obmol.GetAtom(i+1)

            if i == 1:
                matrix.append([0]) #link'd to first atom
                matrix[1].append(current.GetDistance(1))
                continue

            if i == 2:
                matrix.append([1]) #link'd to first atom
                matrix[2].append(current.GetDistance(2))
                matrix[2].append(0)
                matrix[2].append(current.GetAngle(2,1))
                continue

            if i == 3:
                matrix.append([2]) #link'd to most recent
                matrix[3].append(current.GetDistance(3))
                matrix[3].append(1)
                matrix[3].append(current.GetAngle(3, matrix[2][0] + 1))
                matrix[3].append(0)
                torsion = obmol.GetTorsion(4, 3, matrix[2][0] + 1, matrix[2][2] + 1)
                matrix[3].append(torsion)
                continue
                
            #determine closest
            dists = []
            for j in range(3,i):
                atom = obmol.GetAtom(j+1)
                dists.append(current.GetDistance(atom))

            dmin = min(dists)
            dindex = dists.index(dmin) + 3

            #set length values
            matrix.append([dindex])
            matrix[i].append(dmin)

            #set angle values
            matrix[i].append(matrix[dindex][0])
            matrix[i].append(current.GetAngle(dindex + 1, matrix[dindex][0]+1))

            #set torsion values
            matrix[i].append(matrix[dindex][2])
            matrix[i].append(obmol.GetTorsion(i+1, dindex+1, matrix[dindex][0] +1, \
                                matrix[dindex][2]+1))

        return matrix

    def updateCartesian(self):
#    def convertToCartesian(matrix):
        matrix = self.matrix
        size = len(matrix)
        cart = numpy.zeros((size,3),"d")

        for i in range(1, size):
            if i == 1:
                cart[i,0] = matrix[i][1] #distance to the second atom
                continue
            if i == 2:
                temp = numpy.array([matrix[i][1], 0, 0])
                angle = (180 - matrix[i][3])*pi/180
                rot = numpy.array([[cos(angle), sin(angle), 0], \
                                   [-1*sin(angle), cos(angle), 0], \
                                   [ 0, 0, 1]])
                cart[i] = numpy.dot(rot, temp) + cart[1]
                continue

            temp = xvec
            angle = (180 - matrix[i][3]) * pi/180
            rot = numpy.array([[cos(angle), sin(angle), 0], \
                               [-1*sin(angle), cos(angle), 0], \
                               [0, 0, 1]])

            temp2 = numpy.dot(rot, temp)
            
        #right now, we're in the frame of reference of atom1->atom2
        #rotate atom1->atom2 so that it lines up along x-axis
            vec = cart[matrix[i][0]] - cart[matrix[i][2]]
            nvec = sqrt(numpy.dot(vec, vec))
            vec /= nvec

            dot = numpy.dot(vec, xvec)
            if dot != -1 and dot != 1:

            #determine angle and axis that atom1->atom2 is off x-axis
                angle = acos(dot)
                axis = numpy.cross(vec, xvec)
                naxis = sqrt(numpy.dot(axis, axis))
                axis /= naxis

            #rotate so that "x-axis" is alined along atom1->atom2
                c = cos(-1*angle)
                s = sin(-1*angle)
                x = axis[0]
                y = axis[1]
                z = axis[2]

                rot = numpy.array([[c + (1-c)*x*x, (1-c)*x*y - s*z, (1-c)*x*z + s*y], \
                                   [(1-c)*y*x + s*z, c + (1-c)*y*y, (1-c)*y*z - s*x], \
                                   [(1-c)*z*x - s*y, (1-c)*z*y + s*x, c + (1-c)*z*z]])

                temp3 = numpy.dot(rot, temp2)
                temp = temp3  #+ cart[matrix[i][0]] #shift "center" to atom1
            else:
                temp = temp2  #+ cart[matrix[i][0]]

        #now handle torsions!
            axis = cart[matrix[i][2]] - cart[matrix[i][0]]
            vec = cart[matrix[i][4]] - cart[matrix[i][2]]
            nvec = sqrt(numpy.dot(vec, vec))
            torsion = matrix[i][5]*pi/180.0
            
            norm1 = numpy.cross(temp, axis)
            norm2 = numpy.cross(vec, axis)

            nnorm1 = sqrt(numpy.dot(norm1, norm1))
            nnorm2 = sqrt(numpy.dot(norm2, norm2))
            dot = numpy.dot(norm1, norm2)/(nnorm1*nnorm2)
            if dot == -1:
                angle = pi
            elif dot == 1:
                angle = 0
            else:
                angle = acos(dot)

            theta = torsion - angle
            c = cos(-1*theta)
            s = sin(-1*theta)
            naxis = sqrt(numpy.dot(axis, axis))
            x = axis[0]/naxis
            y = axis[1]/naxis
            z = axis[2]/naxis

            rot = numpy.array([[c + (1-c)*x*x, (1-c)*x*y - s*z, (1-c)*x*z + s*y], \
                               [(1-c)*y*x + s*z, c + (1-c)*y*y, (1-c)*y*z - s*x], \
                               [(1-c)*z*x - s*y, (1-c)*z*y + s*x, c + (1-c)*z*z]])


            temp2 = numpy.dot(rot, temp)*matrix[i][1]

            cart[i] = temp2 + cart[matrix[i][0]]

        for i in range(len(cart)):
            atom = self.obmol.GetAtom(i+1)
            atom.SetVector(cart[i,0], cart[i,1], cart[i,2])

    def updateLength(self, atomIndex, index):

        atom = self.obmol.GetAtom(atomIndex + 1)
        self.matrix[atomIndex][0] = index
        dist = atom.GetDistance(index + 1)
        self.matrix[atomIndex][1] = dist
        
        if atomIndex > 1:
            angle, torsion = self.updateAngle(atomIndex, self.matrix[atomIndex][2])
            return dist, angle, torsion
        else:
            return dist, -9999, -9999

    def updateAngle(self, atomIndex, index):
        atom = self.obmol.GetAtom(atomIndex + 1)
        self.matrix[atomIndex][2] = index
        angle = atom.GetAngle(self.matrix[atomIndex][0] +1, index + 1)
        self.matrix[atomIndex][3] = angle

        if atomIndex > 2:
            torsion = self.updateTorsion(atomIndex, self.matrix[atomIndex][4])
            return angle, torsion
        else:
            return angle, -9999

    def updateTorsion(self, atomIndex, index):
        self.matrix[atomIndex][4] = index
        torsion = self.obmol.GetTorsion(atomIndex + 1, self.matrix[atomIndex][0] + 1, \
                        self.matrix[atomIndex][2] + 1, index + 1)
        self.matrix[atomIndex][5] = torsion

        return torsion
