 
# qmforge.py -- code for the mainwindow and the majority of QMForge
#
# Copyright (c) Adam Tenderholt, Stanford University, 2005-2007
#                               a-tenderholt@stanford.edu
#
# This program is free software; you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version with the exceptions noted in the 
# Help dialog.

#created with the help of Qt designer and pyuic

import sys
import logging
from os.path import exists
from time import localtime,asctime,time
from math import sqrt

#import qt modules
from PyQt4 import QtCore,QtGui

#import needed cclib modules
from cclib.parser import ccopen,utils
from cclib.progress import Qt4Progress

import openbabel

from icons import *
from dialogs import *
from fragmentsdialog import *
from aboutbox import *
from logviewer import *
from customdelegate import *
from popwidget import *
from geomwidget import *
from cartwidget import *
from formatter import *
from carttosphere import *

def square(x): return x*x
def sum(x,y): return x+y
table = utils.PeriodicTable()

class QMForge(QtGui.QMainWindow):
    def __init__(self,parent = None):
        QtGui.QMainWindow.__init__(self,parent)
        self.statusBar()
    
        self.image0 = QtGui.QPixmap()
        self.image0.loadFromData(image0_data,"PNG")
        self.image1 = QtGui.QPixmap()
        self.image1.loadFromData(image1_data,"PNG")
        self.image2 = QtGui.QPixmap()
        self.image2.loadFromData(image2_data,"PNG")
        self.image3 = QtGui.QPixmap()
        self.image3.loadFromData(image3_data,"PNG")
        self.imageArrow = QtGui.QPixmap(imageArrow)
        self.imagePlus = QtGui.QPixmap(imagePlus)
        self.imageMinus= QtGui.QPixmap(imageMinus)
        
        self.tabWidget = QtGui.QTabWidget(self)

#work on pop tab
        self.popWidget = PopWidget(self.tabWidget)
        self.tabWidget.addTab(self.popWidget,"Pop. Analyses")

#work on geom tab
        self.geomWidget = GeomWidget(self.tabWidget)
        self.tabWidget.addTab(self.geomWidget,"Geom. Opt.")

#work on cartesian tag
        self.cartWidget = CartWidget(self.tabWidget)
        self.tabWidget.addTab(self.cartWidget,"Cart. Coords.")

#connect slots

#        QtCore.QObject.connect(self.geomWidget, QtCore.SIGNAL("signalUpdateGeometry(int)"), self.cartWidget.slotUpdateGeometry)
#        QtCore.QObject.connect(self.cartWidget, QtCore.SIGNAL("signalUpdateGeometry(int)"), self.geomWidget.slotUpdateGeometry)
        QtCore.QObject.connect(self.tabWidget, QtCore.SIGNAL("currentChanged(int)"),self.slotTabChanged)

        self.setCentralWidget(self.tabWidget)

        self.currentGeom = 0
        self.setupInterface()

    def fileNew(self):
        print "QMForge.fileNew(): Not implemented yet"

    def reset(self):
        
        #clear results
        self.eigens=[]
        self.results=[]
        self.occs=[]
        self.symms=[]
        self.fragnames=[]
        self.unrestricted=False
        
        self.eigensB=[]
        self.resultsB=[]
        self.occsB=[]
        self.symmsB=[]
        
        self.MPAfinished=False
        self.CSPAfinished=False
        self.overlapfinished=False
        
        self.calcMPAAction.setEnabled(self.debug)
        self.calcCSPAAction.setEnabled(self.debug)
        self.calcDOSAction.setEnabled(False)
        self.calcOPAction.setEnabled(self.debug)
        self.calcODOSAction.setEnabled(False)
        
        self.fragView.clear()
        self.fragView.atoms=[]
        self.fragView.addFragment()
        self.fragView.addFragment()
        self.fragView.addFragment()
        self.fragMoveAction.setEnabled(True)
        
        self.aoView.clear()
        
    def open(self,filename):
        
        progress=Qt4Progress("Parsing file...",self)
        viewer = LogViewer(self)
        parser = ccopen(filename,progress,logging.WARNING)
        
        if not parser:
            obconv = openbabel.OBConversion()
            obconv.SetInAndOutFormats(filename[-3:], filename[-3:])
            obmol = openbabel.OBMol()
            retval = obconv.ReadFile(obmol, filename)

            if not retval:
                QtGui.QMessageBox.critical(self,"Error",QtCore.QString("The chosen file doesn't appear to be a calculation logfile or a structure."),"Ok")
                return

            self.cartWidget.setMolecule(obmol)
            self.tabWidget.setCurrentIndex(2)
            self.tabWidget.setTabEnabled(0,False)
            self.tabWidget.setTabEnabled(1,False)
            self.filename = filename
            return

        parser.logger.addHandler(viewer)
        data = parser.parse()
        warnings = viewer.ui.textEdit.toPlainText()
        if len(warnings) > 0:
            viewer.show()

        if hasattr(data, "aonames"):
            data = self.convertToSpherical(data)

        if hasattr(data,"aonames"):
            self.names = data.aonames
        elif hasattr(data,"fonames"):
            self.names = data.fonames
        else:
            self.tabWidget.setCurrentIndex(1)
            self.tabWidget.setTabEnabled(0,False)

        self.popWidget.setData(data)

        if hasattr(data,"atomcoords") and hasattr(data,"atomnos"):
            self.geomWidget.setData(data)
            self.cartWidget.setData(data)

            if len(data.atomcoords) == 1:
                self.tabWidget.setTabEnabled(1,False)
            else:
                self.tabWidget.setCurrentIndex(1)

        self.data=data
        self.filename=filename
        
    def convertToSpherical(self, data):

        ask = True
        starts = []

        for i in range(len(data.aonames)):
            if ask and data.aonames[i].find("XX") > 0:
                retval = QtGui.QMessageBox.question(self, "Cartesian Basis Functions Found", \
                               "The current basis set uses cartesian functions. Do you "+ \
                               "wish to wish to convert them to a spherical basis set?", \
                               QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)

                if retval == QtGui.QMessageBox.Yes:
                    ask = False
                else:
                    return data

            if data.aonames[i].find("XX") > 0:
                starts.append(i)

            data.aonames[i] = data.aonames[i].replace("XX", "S'")
            data.aonames[i] = data.aonames[i].replace("YY", "X2-Y2")
            data.aonames[i] = data.aonames[i].replace("ZZ", "Z2")

        #have we been asked about converting to spherical? if not, then we're
        #already spherical, so just return data
        return data
    
        Dialog = QtGui.QDialog()
        ui = Ui_Dialog()
        ui.setupUi(Dialog)
        Dialog.exec_()

        matrix = numpy.zeros((3,3),"d")
        for i in range(3):
            for j in range(3):
                item = ui.tableWidget.item(i,j)
                val = item.data(QtCore.Qt.DisplayRole).toDouble()[0]
                matrix[i,j] = float(val)

#        matrix = Numeric.zeros((3,3),"d")
#        matrix[:,0] = [1,1,1]
#        matrix[0,1] = sqrt(3/10.0)
#        matrix[1,1] = -1 * sqrt(3/10.0)
#        matrix[0,2] = -0.5 * sqrt(3/5.0)
#        matrix[1,2] = -0.5 * sqrt(3/5.0)
#        matrix[2,2] = 1 * sqrt(3/5.0)

#        print data.mocoeffs[1][35,17]**2, data.mocoeffs[1][35,18]**2, data.mocoeffs[1][35,19]**2
        print reduce(numpy.add, data.mocoeffs[1][35]**2)
        for start in starts:
            for spin in range(len(data.mocoeffs)):
                temp = numpy.inner(data.mocoeffs[spin][:,start:start+3], matrix)
                data.mocoeffs[spin][:,start:start+3] = temp

        print reduce(numpy.add, data.mocoeffs[1][35]**2)
#        print data.mocoeffs[1][35,17]**2, data.mocoeffs[1][35,18]**2, data.mocoeffs[1][35,19]**2

#        for i in range(len(data.mocoeffs[1])):
#            print i, reduce(Numeric.add, data.mocoeffs[1][i]**2)

        return data

    def fileOpen(self):
       
        filters = "Log files (*.log *.out *.adfout)\nAll files (*.*)"
        filename=QtGui.QFileDialog.getOpenFileName(self,"Open Log File...","",filters)
        if not (filename):
            return
        
        self.open(str(filename))
        
    def slotSaveStructure(self):
        namelist = ["XYZ Coordinates","Protein Data Bank", "Z-matrix", "Gaussian Input"]
        extlist = ["xyz","pdb","zmat","com"]
        obformatlist = ["xyz","pdb","fh","com"]
        typelist = ["XYZ","PDB","ZMat","G03"]

        formatdialog = SaveFormatDialog(self,namelist)
        result = formatdialog.exec_()
        if result == QtGui.QDialog.Rejected:
            return

        index = formatdialog.comboBox.currentIndex()
        filters = "%s (*.%s) ;; *"%(namelist[index],extlist[index])

        if hasattr(self,"filename"):
            fileinfo = QtCore.QFileInfo(self.filename)
            ext = fileinfo.suffix()
            rindex = str(self.filename).rfind(ext)
            newfilename = self.filename[:rindex]+extlist[index]

            filename = QtGui.QFileDialog.getSaveFileName(self, "Save %s file"%typelist[index], \
                                    newfilename, filters)

        else:
            newfilename = "newfile." + extlist[index]
            filename = QtGui.QFileDialog.getSaveFileName(self, "Save %s file"%typelist[index], \
                                    newfilename, filters)

        if self.tabWidget.currentIndex() == 1:
            self.geomWidget.glWidget.saveFile(filename,obformatlist[index])
        elif self.tabWidget.currentIndex() == 2:
            self.cartWidget.glWidget.saveFile(filename,obformatlist[index])

    def fileSaveAs(self):
        print "QMForge.fileSaveAs(): Not implemented yet"

    def filePrint(self):
        print "QMForge.filePrint(): Not implemented yet"

    def fileExit(self):
        print "QMForge.fileExit(): Not implemented yet"

    def fragAdd(self):
        self.fragView.addFragment()
        
    def slotUpdateGeom(self,value):
        self.geomWidget.setMolecule(self.data.atomcoords[value],self.data.atomnos)

    def slotUpdateCart(self,value):
        self.cartWidget.setMolecule(self.data.atomcoords[value],self.data.atomnos)
        
        self.cartModel.clear()
        self.cartModel.setHorizontalHeaderLabels(["Element", "x", "y", "z"])
        self.cartView.setColumnWidth(0,55)
        self.cartView.setColumnWidth(1,75)
        self.cartView.setColumnWidth(2,75)
        self.cartView.setColumnWidth(3,75)

        data = self.data
        for i in range(len(data.atomnos)):
                elementItem = QtGui.QStandardItem(table.element[data.atomnos[i]])
                xItem = QtGui.QStandardItem(str(data.atomcoords[value][i][0]))
                yItem = QtGui.QStandardItem(str(data.atomcoords[value][i][1]))
                zItem = QtGui.QStandardItem(str(data.atomcoords[value][i][2]))
                self.cartModel.appendRow([elementItem,xItem,yItem,zItem])

    def slotUpdateCartData(self,topLeft,bottomRight):
        row = topLeft.row()
        col = topLeft.column()

        if row != bottomRight.row() or col != bottomRight.column():
            print "There's an error with editing Cartesian data"
            return

        item = self.cartModel.item(row,col)
        if col == 0:
            tempdata = item.data(QtCore.Qt.DisplayRole).toString()
            data = table.number[str(tempdata)]

        else:
            data, ok = item.data(QtCore.Qt.DisplayRole).toDouble()

        self.cartWidget.updateMolecule(row,col,data)

    def changeCartSelection(self, current, previous):
        indexes = self.selectionModel.selectedIndexes()
        dict = {}
        for index in indexes:
            dict[index.row()] = 1
        self.cartWidget.setSelected(dict.keys())

    def slotSaveFragments(self):
#save fragment list as a text file for easy processing
        try:
            import simplejson
        except ImportError:
            print "simplejson not installed, returning..."
            return

        fragmodel = self.popWidget.fragmodel
        fragnames,fraglists=fragmodel.getFragments()

        for i in range(len(fragnames)):
            fragnames[i] = str(fragnames[i])

        file = open("fraglists.dat","w")
        simplejson.dump(fragnames, file)
        file.write("\n")
        simplejson.dump(fraglists, file)
        file.close()

    def calcMPA(self):
#calculate MPA using cclib and print results using QMForge functions

        aomodel = self.popWidget.aomodel
        fragmodel = self.popWidget.fragmodel

        if  aomodel.rowCount() != 0:
            QtGui.QMessageBox.critical(self,"Error",QtCore.QString("Please assign all orbitals to a fragment before trying a Mulliken Population Analysis."),"Ok")
            return

        if not hasattr(self.data,"moenergies") or not hasattr(self.data,"mocoeffs"):
            QtGui.QMessageBox.critical(self,"Error",QtCore.QString("The logfile is missing information about molecular orbital energies and/or coefficients; therefore, Mulliken Population Analysis is not available."), "Ok")
            return

        if not ( hasattr(self.data,"aooverlaps") or hasattr(self.data,"fooverlaps") ):
            QtGui.QMessageBox.critical(self,"Error",QtCore.QString("The logfile is missing orbital overlap information; therefore, Mulliken Population Analysis is not available."), "Ok")
            return

        fragnames,fraglists=fragmodel.getFragments()

        from cclib.method import MPA
        progress=Qt4Progress("Calculating...",self)
        
        m=MPA(self.data,progress,logging.ERROR)
        m.calculate(fraglists)

        formatter = Formatter(self.filename)
        displayText=formatter.printAllPA(self.data,fraglists,fragnames,m,"MPA")
        self.displayResults(displayText,".txt")
        
        self.MPAfinished=True
        self.CSPAfinished=False
        
    def calcCSPA(self):
#calculate CSPA using cclib and print results using QMForge functions

        aomodel = self.popWidget.aomodel
        fragmodel = self.popWidget.fragmodel

        if aomodel.rowCount() != 0:
            QtGui.QMessageBox.critical(self,"Error",QtCore.QString("Please assign all orbitals to a fragment before trying a C-squared Population Analysis."),"Ok")
            return

        if not hasattr(self.data,"moenergies") or not hasattr(self.data,"mocoeffs"):
            QtGui.QMessageBox.critical(self,"Error",QtCore.QString("The logfile is missing information about molecular orbital energies and/or coefficients; therefore, C-squared Population Analysis is not available."), "Ok")
            return

        fragnames,fraglists=fragmodel.getFragments()

        from cclib.method import CSPA
        progress=Qt4Progress("Calculating...",self)

        m=CSPA(self.data,progress,logging.ERROR)
        m.calculate(fraglists)

        formatter = Formatter(self.filename)
        displayText=formatter.printAllPA(self.data,fraglists,fragnames,m,"CSPA")
        self.displayResults(displayText,".txt")
        
        self.MPAfinished=False
        self.CSPAfinished=True

    def calcFragMPA(self):
#analyze MO's in terms of fragment MOs

        if not hasattr(self, "fragmentdialog"):
            self.fragmentdialog = FragmentDialog(self)
        retval = self.fragmentdialog.exec_()
        if not retval:
            return

        names, paths = self.fragmentdialog.getFragments()
        datas = []
        progress=Qt4Progress("Parsing...",self)
        viewer = LogViewer(self)
        
        for i in range(len(paths)):
            parser = ccopen(paths[i], progress, logging.WARNING)
            parser.logger.addHandler(viewer)
            data = parser.parse()
            data.name = names[i]
            datas.append(data)

        from cclib.method import MPA, FragmentAnalysis
        method = FragmentAnalysis(self.data, None, logging.ERROR)
        method.logger.addHandler(viewer)
        retval = method.calculate(datas)
    
        if retval:

            progress=Qt4Progress("Calculating...",self)

            m=MPA(method,progress,logging.ERROR)
            m.calculate()

            formatter = Formatter(self.filename)
            displayText=formatter.printExtendedPA(self.data, method,m,"Extended MPA")
            self.displayResults(displayText,".txt")

        warnings = viewer.ui.textEdit.toPlainText()

        if len(warnings) > 0:
            viewer.show()

    def calcFragCSPA(self):
#analyze MO's in terms of fragment MOs

        if not hasattr(self, "fragmentdialog"):
            self.fragmentdialog = FragmentDialog(self)
        retval = self.fragmentdialog.exec_()
        if not retval:
            return

        names, paths = self.fragmentdialog.getFragments()
        datas = []
        progress=Qt4Progress("Parsing...",self)
        viewer = LogViewer(self)

        for i in range(len(paths)):
            parser = ccopen(paths[i], progress, logging.WARNING)
            parser.logger.addHandler(viewer)
            data = parser.parse()
            data.name = names[i]
            datas.append(data)

        from cclib.method import CSPA, FragmentAnalysis
        method = FragmentAnalysis(self.data, None, logging.WARNING)
        method.logger.addHandler(viewer)
        retval = method.calculate(datas)
        
        if retval:
            progress=Qt4Progress("Calculating...",self)

            m=CSPA(method,progress,logging.ERROR)
            m.calculate()

            formatter = Formatter(self.filename)
            displayText=formatter.printExtendedPA(self.data, method,m,"Extended CSPA")
            self.displayResults(displayText,".txt")

        warnings = viewer.ui.textEdit.toPlainText()

        if len(warnings) > 0:
            viewer.show()

    def calcFragCDA(self):
#charge decomposition analysis

        self.fragmentdialog = FragmentDialog(self)
        self.fragmentdialog.spinBox.setEnabled(False)
        retval = self.fragmentdialog.exec_()
        if not retval:
            return

        names, paths = self.fragmentdialog.getFragments()
        datas = []
        progress=Qt4Progress("Parsing...",self)
        viewer = LogViewer(self)

        for i in range(len(paths)):
            parser = ccopen(paths[i], progress, logging.WARNING)
            parser.logger.addHandler(viewer)
            data = parser.parse()
            data.name = names[i]
            datas.append(data)

        from cclib.method import CDA
        
        method = CDA(self.data, progress, logging.WARNING)
        method.logger.addHandler(viewer)
        retval = method.calculate(datas)

        if retval:

            formatter = Formatter(self.filename)
            displayText = formatter.printCDA(self.data, names, paths, method)
            self.displayResults(displayText, ".cda")

        warnings = viewer.ui.textEdit.toPlainText()

        if len(warnings) > 0:
            viewer.show()

    def calcOPA(self):
#calculate the overlap populatino

        aomodel = self.popWidget.aomodel
        fragmodel = self.popWidget.fragmodel

        if aomodel.rowCount() != 0:
            QtGui.QMessageBox.critical(self,"Error",QtCore.QString("Please assign all orbitals to a fragment before trying an Overlap Population Analysis."),"Ok")
            return

        fragnames,fraglists=fragmodel.getFragments()

        from cclib.method import OPA

        progress = Qt4Progress("Calculating...",self)
        method = OPA(self.data, progress, logging.ERROR)
        method.calculate(fraglists)

        formatter = Formatter(self.filename)
        displayText = formatter.printAllOP(self.data, fraglists, fragnames, method, "OPA")
        self.displayResults(displayText,".opa")
   
    def calcMBO(self):
      
        from cclib.method import MBO
        from cclib.parser.utils import PeriodicTable

        progress=Qt4Progress("Calculating...",self)
        method = MBO(self.data,progress,logging.ERROR)
        method.calculate()
        isRestricted = ( len(method.fragresults) == 1 )

        displayText="Mayer bond order results of %s\n" % self.filename
        displayText+="Created on %s\n\n"%asctime()
        
#make list of atoms
        atoms=[]
        atomname,orbname = self.names[0].split('_')
        atoms.append(atomname)
        for i in range(len(self.names)):
            atomname,orbname = self.names[i].split('_')
            
            if atomname != atoms[-1]:
                atoms.append(atomname)
            
#write out total MBO
        if isRestricted:
            displayText+="Total\n"
        else:
            displayText+="Total (alpha+beta contributions)\n"
        displayText+="================================\n\n"
        
        base=0
        current=0
        natoms = len(atoms)
        
        while(base<natoms):
        
          displayText+="Total  "
          if natoms-base>=10:
            ncols=10
          else:
            ncols=natoms-base
            
          for i in range(ncols):
            displayText+=" %5s "%(atoms[base+i])
          displayText+="\n"
          displayText+="-------"
          for i in range(ncols):
            displayText+="-------"
          displayText+="\n"
          
          for i in range(base,natoms):
          
            displayText+="%-6s "%(atoms[i])
            for j in range(ncols):
                if isRestricted:
                    displayText+=" %6.3f"%(method.fragresults[0,base+j,i])
                else:
                    displayText+=" %6.3f"%(method.fragresults[0,base+j,i] \
                                            + method.fragresults[1,base+j,i])
                    
            displayText+="\n"
            
          base+=ncols
          displayText+="\n"
    

        if isRestricted:
            
            self.displayResults(displayText,".mbo")
            return

#write out alpha MBO
        displayText+="\n"
        displayText+="Alpha contributions\n"
        displayText+="===================\n\n"
        
        base=0
        current=0
        
        while(base<natoms):
        
          displayText+="Alpha  "
          if natoms-base>=10:
            ncols=10
          else:
            ncols=natoms-base
            
          for i in range(ncols):
            displayText+=" %5s "%(atoms[base+i])
          displayText+="\n"
          displayText+="-------"
          for i in range(ncols):
            displayText+="-------"
          displayText+="\n"
          
          for i in range(base,natoms):
          
            displayText+="%-6s "%(atoms[i])
            for j in range(ncols):
              displayText+=" %6.3f"%(method.fragresults[0,base+j,i])
            displayText+="\n"
            
          base+=ncols
          displayText+="\n"
        
#write out beta MBO
        displayText+="\n"
        displayText+="Beta contributions\n"
        displayText+="==================\n\n"
        
        base=0
        current=0
        
        while(base<natoms):
        
          displayText+="Beta   "
          if natoms-base>=10:
            ncols=10
          else:
            ncols=natoms-base
            
          for i in range(ncols):
            displayText+=" %5s "%(atoms[base+i])
          displayText+="\n"
          displayText+="-------"
          for i in range(ncols):
            displayText+="-------"
          displayText+="\n"
          
          for i in range(base,natoms):
          
            displayText+="%-6s "%(atoms[i])
            for j in range(ncols):
              displayText+=" %6.3f"%(method.fragresults[1,base+j,i])
            displayText+="\n"
            
          base+=ncols
          displayText+="\n"

        self.displayResults(displayText,".mbo")
          
    def displayResults(self,text,suffix):
    
        Dialog = QtGui.QDialog()
        ui = Results_Dialog()
        ui.setupUi(Dialog)
        
        ui.textEdit.setFontFamily("Courier")
        ui.textEdit.setPlainText(text)
        
        Dialog.resize(800,600)
        value=Dialog.exec_()

        if(value):
            
            suggest=self.filename.split(".")[0]+suffix
            qstr=QtGui.QFileDialog.getSaveFileName(self,QtCore.QString("Save results..."),QtCore.QString(suggest),QtCore.QString("*"+suffix+" ;; *"))

            if not qstr:
                return
            
            file=open(qstr,"w")
            file.write(text)
            file.close()
            
    def calcDOS(self):

        lumoindex=self.occs.index(0)
        homoindex=lumoindex-1
        min=self.eigens[homoindex]-10
        max=self.eigens[lumoindex]+10
        
        dos=DOSDialog(self.filename[:-4],".dos",(self.results,self.eigens),self.unrestricted,(self.resultsB,self.eigensB),self.fragnames,min,max)
        dos.exec_loop()
        
    def calcODOS(self):
        
        lumoindex=self.occs.index(0)
        homoindex=lumoindex-1
        min=self.eigens[homoindex]-10
        max=self.eigens[lumoindex]+10
        
        names=[]
        
        for i in range(len(self.fragnames)-1):
            
            for j in range(i+1,len(self.fragnames)):
                
                names.append(self.fragnames[i]+" & "+self.fragnames[j])
                
        dos=DOSDialog(self.filename[:-4],".opd",(self.Oresults,self.eigens),self.unrestricted,(self.OresultsB,self.eigensB),names,min,max,False)
        dos.exec_loop()
        
    #------------------------------------begin calcOverlap-------------------------------------------------
    def calcOverlap(self):
        
        self.frags,self.fragnames=self.fragView.getFragments()
        if(self.overlapfinished):
            displayText=self.printAllOP((self.eigens,self.symms,self.occs,self.Oresults),(self.eigensB,self.symmsB,self.occsB,self.OresultsB),"Overlap Population Analysis")
            self.displayResults(displayText,".opt")
            return
        
        coeffsA,coeffsB=self.parser.getCoeffs()

        #remove empty fragments
        i=0
        while(i<len(self.frags)):
            if(len(self.frags[i])==0):
                self.frags.pop(i)
                self.fragnames.pop(i)
            else:
                i+=1
                
        occsA,occsB=self.parser.getOccs()
        eigensA,eigensB=self.parser.getEigens()
        symmsA,symmsB=self.parser.getSymms()
        
        overlap=self.parser.getOverlap()
        if(len(overlap)==0):
            return
        
        resultsA=[]
        resultsB=[]
        
        sizeA=len(coeffsA)
        sizeB=len(coeffsB)
        steps=sizeA+sizeB
        
        pd=QProgressDialog("Calculating overlap for MO# 0...","Cancel",steps,self,"progress",True)
        pd.setMinimumDuration(1)
        
        pd.setCaption("Overlap Population")
        
        for i in range(sizeA):
            
            pd.setProgress(i)
            if(pd.wasCancelled()):
                return
            
            pd.setLabelText(QString("Calculating overlap for MO# %i..."%(i+1,)))
            
            tempResults=op.results(self.frags,coeffsA[i],overlap)
            resultsA.append(tempResults)
            
        for i in range(sizeB):
            
            pd.setProgress(i+sizeA)
            if(pd.wasCancelled()):
                return
            
            pd.setLabelText(QString("Calculating overlap for beta MO# %i..."%(i+1,)))
            
            tempResults=op.results(self.frags,coeffsB[i],overlap)
            resultsB.append(tempResults)
                
        pd.setProgress(steps)
            
        resultString=""
        resultString=self.printAllOP((eigensA,symmsA,occsA,resultsA),(eigensB,symmsB,occsB,resultsB),"Overlap Population Analysis")
        
        self.displayResults(resultString,".opt")
        
        self.Oresults=resultsA
        self.OresultsB=resultsB
        
        self.eigens=eigensA
        self.eigensB=eigensB
        
        self.symms=symmsA
        self.symmsB=symmsB
        
        self.occs=occsA
        self.occsB=occsB
        
        self.overlapfinished=True
        self.calcODOSAction.setEnabled(True)
        
    #----------------------------end calcOverlap---------------------------------------------------------
    def centerMolecule(self):
        indexes = self.cartWidget.cartView.selectedIndexes()

        if len(indexes) != 1:
            return

        index = indexes[0].row()
        self.cartWidget.cartModel.centerMolecule(index)

    def rotateToX(self):
        indexes = self.cartWidget.cartView.selectedIndexes()
        
        if len(indexes) != 2:
            return

        index1 = indexes[0].row()
        index2 = indexes[1].row()
        if index1 == index2:
            return

        atom1 = pybel.Atom(self.cartWidget.glWidget.obmol.GetAtom(index1+1))
        atom2 = pybel.Atom(self.cartWidget.glWidget.obmol.GetAtom(index2+1))
        
        vec = []
        vec.append(atom2.coords[0] - atom1.coords[0])
        vec.append(atom2.coords[1] - atom1.coords[1])
        vec.append(atom2.coords[2] - atom1.coords[2])

        length = sqrt(vec[0]**2 + vec[1]**2 + vec[2]**2)
        rotvec = [ 0, vec[2]/length, -1*vec[1]/length]

        angle = acos(vec[0]/length)
        self.cartWidget.cartModel.rotateMolecule(angle, [rotvec[0], rotvec[1], rotvec[2]])

    def rotateToY(self):
        indexes = self.cartWidget.cartView.selectedIndexes()
        
        if len(indexes) != 2:
            return

        index1 = indexes[0].row()
        index2 = indexes[1].row()
        if index1 == index2:
            return

        atom1 = pybel.Atom(self.cartWidget.glWidget.obmol.GetAtom(index1+1))
        atom2 = pybel.Atom(self.cartWidget.glWidget.obmol.GetAtom(index2+1))
        
        vec = []
        vec.append(atom2.coords[0] - atom1.coords[0])
        vec.append(atom2.coords[1] - atom1.coords[1])
        vec.append(atom2.coords[2] - atom1.coords[2])

        length = sqrt(vec[0]**2 + vec[1]**2 + vec[2]**2)
        rotvec = [ -1 * vec[2]/length, 0, vec[0]/length ]

        angle = acos(vec[1]/length)
        self.cartWidget.cartModel.rotateMolecule(angle, [rotvec[0], rotvec[1], rotvec[2]])

    def rotateToZ(self):
        indexes = self.cartWidget.cartView.selectedIndexes()
        
        if len(indexes) != 2:
            return

        index1 = indexes[0].row()
        index2 = indexes[1].row()
        if index1 == index2:
            return

        atom1 = pybel.Atom(self.cartWidget.glWidget.obmol.GetAtom(index1+1))
        atom2 = pybel.Atom(self.cartWidget.glWidget.obmol.GetAtom(index2+1))
        
        vec = []
        vec.append(atom2.coords[0] - atom1.coords[0])
        vec.append(atom2.coords[1] - atom1.coords[1])
        vec.append(atom2.coords[2] - atom1.coords[2])

        length = sqrt(vec[0]**2 + vec[1]**2 + vec[2]**2)
        rotvec = [vec[1]/length, -1*vec[0]/length, 0]

        angle = acos(vec[2]/length)
        self.cartWidget.cartModel.rotateMolecule(angle, [rotvec[0], rotvec[1], rotvec[2]])

    def rotateToXY(self):
        indexes = self.cartWidget.cartView.selectedIndexes()
        
        if len(indexes) != 3:
            return

        index1 = indexes[0].row()
        index2 = indexes[1].row()
        index3 = indexes[2].row()

        if index1 == index2:
            return
        if index2 == index3:
            return
        if index1 == index3:
            return

        atom1 = pybel.Atom(self.cartWidget.glWidget.obmol.GetAtom(index1+1))
        atom2 = pybel.Atom(self.cartWidget.glWidget.obmol.GetAtom(index2+1))
        atom3 = pybel.Atom(self.cartWidget.glWidget.obmol.GetAtom(index3+1))
        
        vec1 = []
        vec1.append(atom2.coords[0] - atom1.coords[0])
        vec1.append(atom2.coords[1] - atom1.coords[1])
        vec1.append(atom2.coords[2] - atom1.coords[2])

        vec2 = []
        vec2.append(atom3.coords[0] - atom1.coords[0])
        vec2.append(atom3.coords[1] - atom1.coords[1])
        vec2.append(atom3.coords[2] - atom1.coords[2])

        vec = [ vec1[1]*vec2[2] - vec2[1]*vec1[2], vec1[2]*vec2[0] - vec2[2]*vec1[0], \
                vec1[0]*vec2[1] - vec2[0]*vec1[1] ]

        length = sqrt(vec[0]**2 + vec[1]**2 + vec[2]**2)
        rotvec = [ vec[1]/length, -1*vec[0]/length, 0 ]

        angle = acos(vec[2]/length)
        self.cartWidget.cartModel.rotateMolecule(angle, [rotvec[0], rotvec[1], rotvec[2]])

    def rotateToXZ(self):
        indexes = self.cartWidget.cartView.selectedIndexes()
        
        if len(indexes) != 3:
            return

        index1 = indexes[0].row()
        index2 = indexes[1].row()
        index3 = indexes[2].row()

        if index1 == index2:
            return
        if index2 == index3:
            return
        if index1 == index3:
            return

        atom1 = pybel.Atom(self.cartWidget.glWidget.obmol.GetAtom(index1+1))
        atom2 = pybel.Atom(self.cartWidget.glWidget.obmol.GetAtom(index2+1))
        atom3 = pybel.Atom(self.cartWidget.glWidget.obmol.GetAtom(index3+1))
        
        vec1 = []
        vec1.append(atom2.coords[0] - atom1.coords[0])
        vec1.append(atom2.coords[1] - atom1.coords[1])
        vec1.append(atom2.coords[2] - atom1.coords[2])

        vec2 = []
        vec2.append(atom3.coords[0] - atom1.coords[0])
        vec2.append(atom3.coords[1] - atom1.coords[1])
        vec2.append(atom3.coords[2] - atom1.coords[2])

        vec = [ vec1[1]*vec2[2] - vec2[1]*vec1[2], vec1[2]*vec2[0] - vec2[2]*vec1[0], \
                vec1[0]*vec2[1] - vec2[0]*vec1[1] ]

        length = sqrt(vec[0]**2 + vec[1]**2 + vec[2]**2)
        rotvec = [ -1*vec[2]/length, 0, vec[0]/length ]

        angle = acos(vec[1]/length)
        self.cartWidget.cartModel.rotateMolecule(angle, [rotvec[0], rotvec[1], rotvec[2]])

    def rotateToYZ(self):
        indexes = self.cartWidget.cartView.selectedIndexes()
        
        if len(indexes) != 3:
            return

        index1 = indexes[0].row()
        index2 = indexes[1].row()
        index3 = indexes[2].row()

        if index1 == index2:
            return
        if index2 == index3:
            return
        if index1 == index3:
            return

        atom1 = pybel.Atom(self.cartWidget.glWidget.obmol.GetAtom(index1+1))
        atom2 = pybel.Atom(self.cartWidget.glWidget.obmol.GetAtom(index2+1))
        atom3 = pybel.Atom(self.cartWidget.glWidget.obmol.GetAtom(index3+1))
        
        vec1 = []
        vec1.append(atom2.coords[0] - atom1.coords[0])
        vec1.append(atom2.coords[1] - atom1.coords[1])
        vec1.append(atom2.coords[2] - atom1.coords[2])

        vec2 = []
        vec2.append(atom3.coords[0] - atom1.coords[0])
        vec2.append(atom3.coords[1] - atom1.coords[1])
        vec2.append(atom3.coords[2] - atom1.coords[2])

        vec = [ vec1[1]*vec2[2] - vec2[1]*vec1[2], vec1[2]*vec2[0] - vec2[2]*vec1[0], \
                vec1[0]*vec2[1] - vec2[0]*vec1[1] ]

        length = sqrt(vec[0]**2 + vec[1]**2 + vec[2]**2)
        rotvec = [ 0, vec[2]/length, -1*vec[1]/length ]

        angle = acos(vec[0]/length)
        self.cartWidget.cartModel.rotateMolecule(angle, [rotvec[0], rotvec[1], rotvec[2]])

    def orientXYZ(self):
        if self.tabWidget.currentIndex() == 0:
            self.popWidget.glWidget.orientXYZ()
        elif self.tabWidget.currentIndex() == 1:
            self.geomWidget.glWidget.orientXYZ()
        elif self.tabWidget.currentIndex() == 2:
            self.cartWidget.glWidget.orientXYZ()

    def orientYXZ(self):
        if self.tabWidget.currentIndex() == 0:
            self.popWidget.glWidget.orientYXZ()
        elif self.tabWidget.currentIndex() == 1:
            self.geomWidget.glWidget.orientYXZ()
        elif self.tabWidget.currentIndex() == 2:
            self.cartWidget.glWidget.orientYXZ()

    def orientXZY(self):
        if self.tabWidget.currentIndex() == 0:
            self.popWidget.glWidget.orientXZY()
        elif self.tabWidget.currentIndex() == 1:
            self.geomWidget.glWidget.orientXZY()
        elif self.tabWidget.currentIndex() == 2:
            self.cartWidget.glWidget.orientXZY()

    def orientZXY(self):
        if self.tabWidget.currentIndex() == 0:
            self.popWidget.glWidget.orientZXY()
        elif self.tabWidget.currentIndex() == 1:
            self.geomWidget.glWidget.orientZXY()
        elif self.tabWidget.currentIndex() == 2:
            self.cartWidget.glWidget.orientZXY()

    def helpIndex(self):
        
        QMessageBox.information( self, "Help", "Help documents have not been created yet.\nPlease email the author (atenderholt at users dot sourceforge dot net) with any questions or comments.", QMessageBox.Ok)

    def helpContents(self):
        print "QMForge.helpContents(): Not implemented yet"

    def helpAbout(self):

        Dialog = QtGui.QDialog(self)
        ui = About_Dialog()
        ui.setupUi(Dialog)
        Dialog.show()

    def addSlot(self):
        if self.tabWidget.currentIndex() == 0:
            self.popWidget.fragmodel.addFragment()           
        elif self.tabWidget.currentIndex() == 2:
            self.cartWidget.addAtom()

    def removeSlot(self):
        if self.tabWidget.currentIndex() == 0:
            self.popWidget.fragview.slotRemoveFragments()
        elif self.tabWidget.currentIndex() == 2:
            self.cartWidget.removeAtom()

    def __tr(self,s,c = None):
        return qApp.translate("QMForge",s,c)

    def slotTabChanged(self,value):

        if value == 0:
            self.saveStructureAction.setEnabled(False)
            
            self.MPAAction.setEnabled(True)
            self.CSPAAction.setEnabled(True)
            self.MBOAction.setEnabled(True)
            self.OPAAction.setEnabled(True)

            self.centerAction.setEnabled(False)
            self.addFragmentAction.setEnabled(True)
            self.removeFragmentAction.setEnabled(True)

            self.centerAction.setEnabled(False)
            self.rotateToXAction.setEnabled(False)
            self.rotateToYAction.setEnabled(False)
            self.rotateToZAction.setEnabled(False)

            self.rotateToXYAction.setEnabled(False)
            self.rotateToXZAction.setEnabled(False)
            self.rotateToYZAction.setEnabled(False)

        elif value == 1 or value == 2:
            self.saveStructureAction.setEnabled(True)

            self.MPAAction.setEnabled(False)
            self.CSPAAction.setEnabled(False)
            self.MBOAction.setEnabled(False)
            self.OPAAction.setEnabled(False)

        if value == 1:
            num = self.cartWidget.spinBox.value()
            self.geomWidget.slotUpdateGeometry(num)

            self.centerAction.setEnabled(False)
            self.rotateToXAction.setEnabled(False)
            self.rotateToYAction.setEnabled(False)
            self.rotateToZAction.setEnabled(False)

            self.rotateToXYAction.setEnabled(False)
            self.rotateToXZAction.setEnabled(False)
            self.rotateToYZAction.setEnabled(False)

            self.addFragmentAction.setEnabled(False)
            self.removeFragmentAction.setEnabled(False)

        if value == 2:
            num = self.geomWidget.spinBox.value()
            self.cartWidget.slotUpdateGeometry(num)

            self.centerAction.setEnabled(True)
            self.rotateToXAction.setEnabled(True)
            self.rotateToYAction.setEnabled(True)
            self.rotateToZAction.setEnabled(True)

            self.rotateToXYAction.setEnabled(True)
            self.rotateToXZAction.setEnabled(True)
            self.rotateToYZAction.setEnabled(True)

            self.addFragmentAction.setEnabled(True)
            self.removeFragmentAction.setEnabled(True)


    def setupInterface(self):

        fileMenu = self.menuBar().addMenu(self.tr("&File"))

        openAction = fileMenu.addAction(QtGui.QIcon(self.image1),self.tr("&Open..."))
        openAction.setShortcut(QtGui.QKeySequence("Ctrl+O"))

        saveStructureAction = fileMenu.addAction(QtGui.QIcon(self.image2),self.tr("&Save structure..."))
        saveStructureAction.setShortcut(QtGui.QKeySequence("Ctrl+S"))
        saveFragmentsAction = fileMenu.addAction(QtGui.QIcon(), \
                                self.tr("Save fragments..."))

        exitAction = fileMenu.addAction(self.tr("E&xit"))
        exitAction.setShortcut(QtGui.QKeySequence("Ctrl+Q"))

        calculateMenu = self.menuBar().addMenu(self.tr("&Calculate"))
        MPAAction = calculateMenu.addAction(self.tr("&Mulliken..."))
        MPAAction.setShortcut(QtGui.QKeySequence("Ctrl+M"))
        CSPAAction = calculateMenu.addAction(self.tr("&C-squared..."))
        calculateMenu.addSeparator()
        subMenu = calculateMenu.addMenu(self.tr("Fragment Analysis"))
        OPAAction = calculateMenu.addAction(self.tr("&Overlap Population..."))
        MBOAction = calculateMenu.addAction(self.tr("Mayer's &Bond Orders..."))

        fragMPAAction = subMenu.addAction(self.tr("Mulliken..."))
        fragCSPAAction = subMenu.addAction(self.tr("C-squared..."))
        fragCDAAction = subMenu.addAction(self.tr("Charge Decomposition..."))

        modifyMenu = self.menuBar().addMenu(self.tr("Modify"))
        centerAction = modifyMenu.addAction(self.tr("&Center Molecule"))
        centerAction.setEnabled(False)
        modifyMenu.addSeparator()
#rotate to axis actions
        rotateToXAction = modifyMenu.addAction(self.tr("Rotate to X"))
        rotateToXAction.setEnabled(False)
        rotateToYAction = modifyMenu.addAction(self.tr("Rotate to Y"))
        rotateToYAction.setEnabled(False)
        rotateToZAction = modifyMenu.addAction(self.tr("Rotate to Z"))
        rotateToZAction.setEnabled(False)
        modifyMenu.addSeparator()
#rotate to plane actions
        rotateToXYAction = modifyMenu.addAction(self.tr("Rotate to XY"))
        rotateToXYAction.setEnabled(False)
        rotateToXZAction = modifyMenu.addAction(self.tr("Rotate to XZ"))
        rotateToXZAction.setEnabled(False)
        rotateToYZAction = modifyMenu.addAction(self.tr("Rotate to YZ"))
        rotateToYZAction.setEnabled(False)

        viewMenu = self.menuBar().addMenu(self.tr("&View"))
        orientXYZAction = viewMenu.addAction(self.tr("Orient View to XYZ..."))
        orientYXZAction = viewMenu.addAction(self.tr("Orient View to YXZ..."))
        orientXZYAction = viewMenu.addAction(self.tr("Orient View to XZY..."))
        orientZXYAction = viewMenu.addAction(self.tr("Orient View to ZXY..."))

        helpMenu = self.menuBar().addMenu(self.tr("&Help"))
        aboutAction = helpMenu.addAction(self.tr("&About"))
        
        self.connect(openAction, QtCore.SIGNAL("triggered()"), self.fileOpen)
        self.connect(saveStructureAction, QtCore.SIGNAL("triggered()"), self.slotSaveStructure)
        self.connect(saveFragmentsAction, QtCore.SIGNAL("triggered()"), self.slotSaveFragments)
        self.connect(exitAction, QtCore.SIGNAL("triggered()"), QtGui.qApp, QtCore.SLOT("quit()"))

        self.connect(MPAAction, QtCore.SIGNAL("triggered()"), self.calcMPA)
        self.connect(CSPAAction, QtCore.SIGNAL("triggered()"), self.calcCSPA)
        self.connect(fragMPAAction, QtCore.SIGNAL("triggered()"), self.calcFragMPA)
        self.connect(fragCSPAAction, QtCore.SIGNAL("triggered()"), self.calcFragCSPA)
        self.connect(fragCDAAction, QtCore.SIGNAL("triggered()"), self.calcFragCDA)
        self.connect(MBOAction, QtCore.SIGNAL("triggered()"), self.calcMBO)
        self.connect(OPAAction, QtCore.SIGNAL("triggered()"), self.calcOPA)

        self.connect(centerAction, QtCore.SIGNAL("triggered()"), self.centerMolecule)
        self.connect(rotateToXAction, QtCore.SIGNAL("triggered()"), self.rotateToX)
        self.connect(rotateToYAction, QtCore.SIGNAL("triggered()"), self.rotateToY)
        self.connect(rotateToZAction, QtCore.SIGNAL("triggered()"), self.rotateToZ)
        self.connect(rotateToXYAction, QtCore.SIGNAL("triggered()"), self.rotateToXY)
        self.connect(rotateToXZAction, QtCore.SIGNAL("triggered()"), self.rotateToXZ)
        self.connect(rotateToYZAction, QtCore.SIGNAL("triggered()"), self.rotateToYZ)

        self.connect(orientXYZAction, QtCore.SIGNAL("triggered()"), self.orientXYZ)
        self.connect(orientYXZAction, QtCore.SIGNAL("triggered()"), self.orientYXZ)
        self.connect(orientXZYAction, QtCore.SIGNAL("triggered()"), self.orientXZY)
        self.connect(orientZXYAction, QtCore.SIGNAL("triggered()"), self.orientZXY)

        self.connect(aboutAction, QtCore.SIGNAL("triggered()"), self.helpAbout)

        toolBar = self.addToolBar("")

        toolBar.insertAction(None,openAction)
        addFragmentAction = toolBar.addAction(QtGui.QIcon(self.imagePlus),QtCore.QString("Add Fragment"))
        removeFragmentAction = toolBar.addAction(QtGui.QIcon(self.imageMinus),QtCore.QString("Remove Fragment"))

        self.connect(addFragmentAction, QtCore.SIGNAL("triggered()"), self.addSlot) 
        self.connect(removeFragmentAction, QtCore.SIGNAL("triggered()"), self.removeSlot)

#make global stuff 
        self.saveStructureAction = saveStructureAction

        self.MPAAction = MPAAction
        self.CSPAAction = CSPAAction
        self.OPAAction = OPAAction
        self.MBOAction = MBOAction

        self.centerAction = centerAction

        self.addFragmentAction = addFragmentAction
        self.removeFragmentAction = removeFragmentAction

        self.rotateToXAction = rotateToXAction
        self.rotateToYAction = rotateToYAction
        self.rotateToZAction = rotateToZAction

        self.rotateToXYAction = rotateToXYAction
        self.rotateToXZAction = rotateToXZAction
        self.rotateToYZAction = rotateToYZAction

if __name__=='__main__':
    app = QtGui.QApplication(sys.argv)
    window = QMForge()
    window.resize(800,600)
    window.show()
    if len(sys.argv)==2 and exists(sys.argv[1]):
        window.open(sys.argv[1])
        
    sys.exit(app.exec_())

